function d = moveMask(c, flow, clusterQ)

% Copyright 2018 Laurent Hoeltgen <contact@laurenthoeltgen.name>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision on: 25.05.2018 13:26

[nr, nc, ~, nd] = size(flow);

d = zeros(nr, nc, nd + 1);
d(:,:,1) = c;

nCl = max(1,sum(abs(c(:)) > 0.01));

for ii = 2:(nd + 1)    
    % Shift mask points (currently w/ rounding and w/o registration) 
    tmp = ftransp(d(:,:,ii-1), flow(:,:,1,ii-1), flow(:,:,2,ii-1), true);
    
    if clusterQ
        %% Cluster (not recommended)
        
        [pax, pay] = find(tmp);
        pa = sub2ind(size(tmp), pax, pay);
        va = tmp(pa);
        
        [idx, C] = kmeans([pax, pay, va], min([nCl, floor(0.25*numel(va)), 7500]));
        
        Crx = round(C(:,1));
        Cry = round(C(:,2));
        out = zeros(size(tmp));
        
        for jj = 1:max(idx)
            out(Crx(jj),Cry(jj)) = sum(va(idx(jj))); % idx(ii);
        end
    else
        % Cut of mask values > 1
        tmp = round(tmp);
        tmp(tmp > 1) = 1;
        out = tmp;
    end
    
    % Save clustered mask
    d(:,:,ii) = out;
end

end